export declare enum NetworkType {
    test = "test",
    prod = "prod"
}
export declare enum Currency {
    unknown = "unknown",
    dollars = "dollars",
    ethers = "ethers",
    bitcoins = "bitcoins"
}
export declare enum Imparter {
    unknown = "unknown",
    ohledger = "ohledger",
    ohledgerWeb3 = "ohledger-web3",
    ohledgerSocial = "ohledger-social",
    ethWeb3 = "eth-web3",
    btcManual = "btc-manual"
}
export declare enum Social {
    unknown = "unknown",
    microsoft = "microsoft",
    google = "google"
}
export declare const CURRENCY_BY_IMPARTER: {
    [what in Imparter]: Currency;
};
export declare const NETWORKS_BY_IMPARTER: {
    [which in NetworkType]: {
        [what in Imparter]: string;
    };
};
export interface PaymentsInfo {
    wallet: {
        [which in Imparter]: boolean;
    };
    isOnLedger: {
        [which in Imparter]: boolean;
    };
    payerAddress: {
        [which in Imparter]: string | null;
    };
    payerPrivateKey: {
        [which in Imparter]: string | null;
    };
    payerSignature: {
        [which in Imparter]: string | null;
    };
    messageToSign: {
        [which in Imparter]: string | null;
    };
    currentImparter: Imparter;
    currentCurrency: Currency;
    currentSocial: Social;
    ordinal: number;
    loginElement?: IPay2MyAppLogin | null;
    pendingTransaction: IPay2MyAppPendingTransactionEvent;
    skuAuthorizations: {
        [sku: string]: boolean;
    };
    skuComponents: {
        [sku: string]: IPay2MyAppAppsell;
    };
}
/**
 * Event fired by pay2myapp-appsell as a custom event: "pay2myapp-appsell-sku-clicked"
 *
 * The event fired by an pay2myapp-appsell component when an appsell
 * SKU deemed authorized by overhide is clicked by the user.
 *
 * Usually safest to route state-changes in response to this
 * event via the back-end, and validate authorizations.
 *
 * All necessary information to validate is provided in this
 * event.
 *
 * Passing the `asOf` timestamp to your back-end is an important optimization.  The overhide
 * services already checked these transactions as part of this front-end work.  The `asOf` timestamp
 * ensures we re-load these resutls from cache and do not get rate-limited in the back-end.
 */
export interface IPay2MyAppSkuClickedEvent {
    sku: string;
    message: string;
    signature: string;
    from: string;
    to: string;
    currency: Currency;
    isTest: boolean;
    asOf: string;
}
/**
 * Event fired by pay2myapp-appsell as a custom event: "pay2myapp-appsell-topup-outstanding"
 *
 * An event fired by an pay2myapp-appsell component when
 * there was an authorization attempt but insufficient funds
 * to authorize.
 *
 * This even contains the outstanind topup funds required.
 */
export interface IPay2MyAppSkuTopupOutstandingEvent {
    sku: string;
    topup: number;
}
/**
 * Event fired by pay2myapp-hub as a custom event: "pay2myapp-hub-sku-authorization-changed"
 *
 * Indicated a change in authorization status.
 */
export interface IPay2MyAppSkuAuthorizationChangedEvent {
    isAuthorized: boolean;
}
/**
 * Event fired by pay2myapp-hub as a custom event: "pay2myapp-hub-pending-transaction"
 *
 * Fired when we have a pending transaction.  We're waiting for a transaction to finish.
 *
 * This should be useful for spinners on custom pay2myapp-appsell components.
 *
 * All pay2myapp-appsell components should spin when a transaction is pending.
 */
export interface IPay2MyAppPendingTransactionEvent {
    isPending: boolean;
    currency: string | null;
}
export interface IPay2MyAppAppsell {
    setHub(hub: IPay2MyAppHub): void;
    click(): void;
}
export interface IPay2MyAppLogin {
    setHub(hub: IPay2MyAppHub): void;
    close(): void;
    open(): Promise<void>;
}
export interface IPay2MyAppStatus {
    setHub(hub: IPay2MyAppHub): void;
}
export interface IPay2MyAppHub {
    init: () => void;
    getNetwork: (imparter: Imparter) => string;
    getNetworkType: () => NetworkType;
    setError: (error: string) => void;
    setSecretKey: (imparter: Imparter, newKey: string) => Promise<boolean>;
    setAddress: (imparter: Imparter, newAddress: string) => Promise<boolean>;
    generateNewKeys: (imparter: Imparter) => void;
    setCurrentImparter: (imparter: Imparter) => Promise<boolean>;
    setCurrentSocial: (social: Social) => void;
    isAuthenticated: (impater: Imparter) => boolean;
    getTally: (to: string, tallyMinutes: number | null) => Promise<{
        tally: number | null;
        asOf: string | null;
    }>;
    getOutstanding: (costInDollars: number, to: string, tallyMinutes: number | null) => Promise<{
        delta: number | null;
        asOf: string | null;
    }>;
    topUp: (amountDollars: number, toAddress: string) => Promise<boolean>;
    getUrl: (impater: Imparter) => string;
    clear: (imparter: Imparter) => void;
    getInfo: () => PaymentsInfo;
    setLoginElement: (element?: IPay2MyAppLogin | null) => void;
    refresh: (imparter: Imparter | null) => void;
    setSkuAuthorized: (sku: string, authorized: boolean) => void;
    isSkuAuthorized: (sku: string) => boolean;
    setComponentForSku: (sku: string, component: IPay2MyAppAppsell) => void;
    getComponentForSku: (sku: string) => IPay2MyAppAppsell | null;
    logout: () => void;
}
//# sourceMappingURL=definitions.d.ts.map